package gift_collect

import (
	"context"
	"github.com/robfig/cron/v3"
	"github.com/zeromicro/go-zero/core/logx"
	pbMq "live-service/app/pb/mq"
	"live-service/app/user_center/rpc/internal/svc"
)

var Service *GiftCollector

type (
	GiftData struct {
		Id     int64   // 礼物ID
		Name   string  // 礼物名称
		Price  float64 // 礼物单价(rmb)
		IsPaid bool    // 是否收费礼物
	}
	GiftCollector struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		cron   *cron.Cron

		collectorMapper map[string]CollectorFunc

		// cacheGift 礼物缓存 platform:giftId:giftData
		cacheGift map[string]map[int64]GiftData

		logx.Logger
	}

	CollectorFunc func(ctx context.Context, svcCtx *svc.ServiceContext) (ret map[int64]GiftData, err error)
)

func InitCollector(svcCtx *svc.ServiceContext) {
	ctx := context.Background()
	Service = &GiftCollector{
		ctx:    context.Background(),
		svcCtx: svcCtx,
		cron:   cron.New(cron.WithSeconds()),
		collectorMapper: map[string]CollectorFunc{
			pbMq.Platform_name[int32(pbMq.Platform_bilibili)]: GiftCollectorBilibili,
		},
		Logger: logx.WithContext(ctx),
	}

	if !svcCtx.GameConfig.GiftCollector.Enabled {
		return
	}
	Service.startJob()
}

func (c *GiftCollector) startJob() {
	cfg := c.svcCtx.GameConfig.GiftCollector
	// 先执行一次
	c.collectGift(cfg.Platforms...)

	_, _ = c.cron.AddFunc(cfg.Cron.CollectGift, func() {
		c.collectGift(cfg.Platforms...)
	})
	c.cron.Start()
}

func (c *GiftCollector) collectGift(platforms ...string) {
	for _, platform := range platforms {
		if handle, ok := c.collectorMapper[platform]; ok {
			ret, err := handle(c.ctx, c.svcCtx)
			if err != nil {
				// TODO 打印
				continue
			}
			// caching
			c.cacheGift[platform] = ret
		}
	}
}

// GetCacheByPlatform 获取缓存中的礼物信息
func (c *GiftCollector) GetCacheByPlatform(platform string) (map[int64]GiftData, bool) {
	data, ok := c.cacheGift[platform]
	return data, ok
}
