package gift_collect

import (
	"context"
	"encoding/json"
	"git.noahlan.cn/northlan/ntools-go/gorm-zero/gormx"
	"git.noahlan.cn/northlan/ntools-go/uuid"
	"github.com/pkg/errors"
	"gorm.io/gorm"
	"io/ioutil"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/svc"
	"net/http"
	"strconv"
)

const (
	CoinTypeGold   = "gold"
	CoinTypeSilver = "silver"
)

type (
	bilibiliGift struct {
		Id       int64  `json:"id"`        // 礼物ID
		Name     string `json:"name"`      // 礼物名称
		Price    int64  `json:"price"`     // 礼物单价
		CoinType string `json:"coin_type"` // 瓜子类型 gold金瓜子 silver银瓜子
		ImgBasic string `json:"img_basic"` // 礼物图
	}
)

func isPaidGift(coinType string) bool {
	if coinType == CoinTypeGold {
		return true
	}
	if coinType == CoinTypeSilver {
		return false
	}
	return false
}

func GiftCollectorBilibili(ctx context.Context, svcCtx *svc.ServiceContext) (ret map[int64]GiftData, err error) {
	const platform = "bilibili"
	radio, _ := svcCtx.GameConfig.Coin.GiftToRMB[platform]

	// 1. 读取API
	httpResp, err := http.Get("https://api.live.bilibili.com/xlive/web-room/v1/giftPanel/giftConfig?platform=app")
	if err != nil {
		return
	}
	resBody, err := ioutil.ReadAll(httpResp.Body)
	if err != nil {
		return
	}
	var responseData struct {
		Code    int    `json:"code"`
		Message string `json:"message"`
		Data    struct {
			List []bilibiliGift `json:"list"`
		} `json:"data"`
	}
	err = json.Unmarshal(resBody, &responseData)
	if err != nil {
		return
	}
	// 2. 事务分批入库 限定每次分批入库数量
	batchSize := 50
	dataLen := len(responseData.Data.List)
	batches := dataLen / batchSize
	if dataLen%batchSize != 0 {
		batches += 1
	}
	ret = make(map[int64]GiftData)
	for i := 0; i < batches; i++ {
		_ = svcCtx.GiftModel.TransactCtx(ctx, nil, func(tx *gorm.DB) error {
			lenJ := (i + 1) * batchSize
			if lenJ > dataLen {
				lenJ = dataLen
			}
			for j := i * batchSize; j < lenJ; j++ {
				gift := responseData.Data.List[j]

				dbGift := &model.Gift{
					Id:       uuid.NextId(),
					GiftId:   strconv.FormatInt(gift.Id, 10),
					GiftName: gift.Name,
					Pic:      gift.ImgBasic,
					Platform: platform,
				}
				giftData := GiftData{
					Id:   gift.Id,
					Name: gift.Name,
				}
				if isPaidGift(gift.CoinType) {
					dbGift.PPricePaid = gift.Price
					dbGift.Price = float64(gift.Price) * float64(radio)

					giftData.IsPaid = true
					giftData.Price = dbGift.Price
				} else {
					dbGift.PPriceFree = gift.Price

					giftData.IsPaid = false
					giftData.Price = float64(dbGift.PPriceFree)
				}
				ret[giftData.Id] = giftData

				if err = svcCtx.GiftModel.Update(ctx, tx, dbGift); err != nil {
					if errors.Is(err, gormx.ErrRowsAffectedZero) {
						err = nil
						_ = svcCtx.GiftModel.Insert(ctx, tx, dbGift)
					}
				}
			}
			return nil
		})
	}
	return
}
