package rank

import (
	"context"
	"git.noahlan.cn/northlan/ntools-go/gorm-zero/gormx"
	"github.com/jinzhu/now"
	"github.com/pkg/errors"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	pbVars "live-service/app/pb/vars"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/common/coin_manager"
	"live-service/common/nerr"
	"time"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type RankPvpSubmitLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewRankPvpSubmitLogic(ctx context.Context, svcCtx *svc.ServiceContext) *RankPvpSubmitLogic {
	return &RankPvpSubmitLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *RankPvpSubmitLogic) RankPvpSubmit(in *pb.RankPvpSubmitReq) (*pb.RankPvpSubmitResp, error) {
	resp := &pb.RankPvpSubmitResp{
		Items: make([]*pb.RankPvpSubmitResp_Item, 0),
	}
	if in.AllRankType {
		if err := l.svcCtx.Db.Transaction(func(tx *gorm.DB) error {
			for _, rankType := range model.AllRankType() {
				if item, err := l.rankByType(rankType, tx); err != nil {
					return err
				} else {
					resp.Items = append(resp.Items, item)
				}
			}
			return nil
		}); err != nil {
			return nil, errors.Wrap(nerr.NewWithErr(err), "结算排行榜失败")
		}
	} else {
		// rankType判断
		var existsRankType bool
		for _, rankType := range model.AllRankType() {
			if rankType == in.RankType {
				existsRankType = true
				break
			}
		}
		if !existsRankType {
			return nil, errors.Wrap(nerr.NewError(nerr.RankNotExists, "排行榜类型不存在（不对）"), "排行榜类型不存在（不对）")
		}

		if item, err := l.rankByType(in.RankType, nil); err != nil {
			return nil, errors.Wrap(nerr.NewWithErr(err), "结算排行榜失败")
		} else {
			resp.Items = append(resp.Items, item)
		}
	}
	return resp, nil
}

func (l *RankPvpSubmitLogic) rankByType(rankType pbVars.RankType, tx *gorm.DB) (*pb.RankPvpSubmitResp_Item, error) {
	if !l.svcCtx.GameConfig.Rank.Enabled {
		return nil, nerr.NewError(nerr.RankNotEnabled, "排行榜功能未开启")
	}
	// . 获取对应榜单列表(用户)
	// . 称号奖励+弹币奖励
	// . 清理对应榜的数据(数据库+内存rank)
	rankTypeInt := int32(rankType)
	resp := &pb.RankPvpSubmitResp_Item{
		RankType: rankType,
		Results:  make([]*pb.RankPvpSubmitResp_Result, 0),
	}
	rsCfg := l.svcCtx.GameConfig.Rank.Submit
	cfg, ok := rsCfg[rankTypeInt]
	if !ok {
		cfg = rsCfg[0]
	}
	titleCfg, _ := l.svcCtx.GameConfig.Title.FindTitleCfg(cfg.TitleId)
	titleExists := titleCfg != nil

	rmbToCoin := l.svcCtx.GameConfig.Coin.RMBToCoin

	if err := gormx.WithTx(l.ctx, l.svcCtx.Db, tx).Transaction(func(tx *gorm.DB) error {
		rankRange := Service.RangeRankByType(rankType, model.MaxRankN)
		if len(rankRange.Items) == 0 {
			return nil
		}
		for i, item := range rankRange.Items {
			tmpItem := &pb.RankPvpSubmitResp_Result{
				UserId:   item.UserId,
				Username: item.Username,
				Avatar:   item.Avatar,
			}
			if i == 0 {
				// 榜首 称号奖励 从今日持续到本周日早
				if titleExists {
					durationHour := now.With(now.Sunday()).BeginningOfDay().Sub(now.BeginningOfDay()).Hours()
					err := l.svcCtx.UserTitleModel.Addon(l.ctx, tx, item.UserId, titleCfg.Id, titleCfg.Type,
						time.Duration(durationHour)*time.Hour, false)
					if err != nil {
						return nerr.NewWithErr(err)
					}
					tmpItem.Title = titleCfg.Id
				}
			}
			var coinAddon int64
			if i < len(cfg.Best) {
				// 前N奖励
				coinAddon = decimal.NewFromFloat(rmbToCoin).Mul(decimal.NewFromFloat(cfg.Best[i])).Round(0).IntPart()
			} else {
				// 安慰奖
				coinAddon = decimal.NewFromFloat(rmbToCoin).Mul(decimal.NewFromFloat(cfg.Consolation)).Round(0).IntPart()
			}
			_, err := l.svcCtx.CoinManager.ChangeCoin(l.ctx, tx, &coin_manager.ChangeCoinReq{
				UserId: item.UserId,
				Change: coinAddon,
			}, pbVars.UserCoinChangedReason_EventRankSubmit)
			if err != nil {
				return nerr.NewWithErr(err)
			}
			tmpItem.Coin = coinAddon
			// . 清理
			if err = l.svcCtx.StatisticsPvpModel.CleanByType(l.ctx, tx, model.ScoreTypeByRankType(rankType)); err != nil {
				if !errors.Is(err, gormx.ErrRowsAffectedZero) {
					return nerr.NewWithErr(err)
				}
			}
			if !Service.CleanByType(rankType) {
				return nerr.NewError(nerr.RankSubmitErr, "清理内存排行榜数据失败")
			}
			resp.Results = append(resp.Results, tmpItem)
		}
		return nil
	}); err != nil {
		return nil, nerr.NewWithErr(err)
	}
	return resp, nil
}
