package user

import (
	"context"
	"gorm.io/gorm"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"
	"live-service/common/timex"
	"time"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetUserDetailsLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetUserDetailsLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetUserDetailsLogic {
	return &GetUserDetailsLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// getUserDetails 获取用户详细信息
func (l *GetUserDetailsLogic) GetUserDetails(in *pb.UserIdReq) (*pb.UserDetailsResp, error) {
	resp := &pb.UserDetailsResp{
		UserId: in.UserId,
	}
	// 贵族
	var nobilityLevel int32
	if dbNobility, err := l.svcCtx.UserNobilityModel.FindOne(l.ctx, nil, in.UserId); err == nil {
		nobilityLevel = int32(dbNobility.NobilityLevel)
		// 判断过期
		if time.Now().After(dbNobility.EndTime) {
			if !dbNobility.Forever {
				nobilityLevel = 0
			}
		}
	}
	resp.NobilityLevel = nobilityLevel
	// 金币
	var coin int64
	if dbCoin, err := l.svcCtx.CoinManager.GetUserCoin(l.ctx, in.UserId); err == nil {
		coin = dbCoin
	}
	resp.Coin = coin

	nowTime := time.Now()

	_ = l.svcCtx.Db.Transaction(func(tx *gorm.DB) error {
		if zhgUserDetails, err := l.svcCtx.ZhgUserDetailsModel.FindOne(l.ctx, tx, in.UserId); err == nil {
			if dbTitle, err := l.svcCtx.UserTitleModel.FindOneByUserIdTitleId(l.ctx, tx, in.UserId, zhgUserDetails.Title); err == nil {
				if !timex.DayExpire(nowTime, dbTitle.EndTime, bool(dbTitle.Forever)) {
					var name string
					if cfg, err := l.svcCtx.GameConfig.Title.FindTitleCfg(dbTitle.TitleId); err == nil {
						name = cfg.Name
					}
					resp.CurrentTitle = &pb.UserDetailsResp_TitleItem{
						Id:     dbTitle.TitleId,
						Name:   name,
						Sort:   int32(dbTitle.Sort),
						Remain: timex.DayRemain(nowTime, dbTitle.EndTime, bool(dbTitle.Forever)),
					}
				}
			}
			if dbElite, err := l.svcCtx.UserEliteModel.FindOneByUserIdEliteId(l.ctx, tx, in.UserId, zhgUserDetails.Elite); err == nil {
				if !timex.DayExpire(nowTime, dbElite.EndTime, bool(dbElite.Forever)) {
					var sort int32 = 2
					if cfg, err := l.svcCtx.GameConfig.Elite.FindEliteCfg(dbElite.EliteId); err == nil {
						sort = cfg.Sort
					}
					resp.CurrentElite = &pb.UserDetailsResp_EliteItem{
						Id:     dbElite.EliteId,
						Sort:   sort,
						Remain: timex.DayRemain(nowTime, dbElite.EndTime, bool(dbElite.Forever)),
					}
				}
			}
		}
		return nil
	})

	// 拥有 称号/精英单位 列表
	resp.Elites = make([]*pb.UserDetailsResp_EliteItem, 0)
	if dbElites, err := l.svcCtx.UserEliteModel.FindByUserId(l.ctx, nil, in.UserId); err == nil {
		for _, item := range dbElites {
			if !timex.DayExpire(nowTime, item.EndTime, bool(item.Forever)) {
				var sort int32 = 2
				if cfg, err := l.svcCtx.GameConfig.Elite.FindEliteCfg(item.EliteId); err == nil {
					sort = cfg.Sort
				}
				resp.Elites = append(resp.Elites, &pb.UserDetailsResp_EliteItem{
					Id:     item.EliteId,
					Sort:   sort,
					Remain: timex.DayRemain(nowTime, item.EndTime, bool(item.Forever)),
				})
			}
		}
	}

	resp.Titles = make([]*pb.UserDetailsResp_TitleItem, 0)
	if dbTitles, err := l.svcCtx.UserTitleModel.FindByUserId(l.ctx, nil, in.UserId); err == nil {
		for _, item := range dbTitles {
			if !timex.DayExpire(nowTime, item.EndTime, bool(item.Forever)) {
				var name string
				if cfg, err := l.svcCtx.GameConfig.Title.FindTitleCfg(item.TitleId); err == nil {
					name = cfg.Name
				}
				resp.Titles = append(resp.Titles, &pb.UserDetailsResp_TitleItem{
					Id:     item.TitleId,
					Name:   name,
					Sort:   int32(item.Sort),
					Remain: timex.DayRemain(nowTime, item.EndTime, bool(item.Forever)),
				})
			}
		}
	}
	return resp, nil
}
